--
-- Created by IntelliJ IDEA.
-- User: admin
-- Date: 5/07/2014
-- Time: 5:23 PM
-- To change this template use File | Settings | File Templates.
-- =D
--

local Ace = LibStub("AceAddon-3.0")
local Comergy2 = Ace:NewAddon("Comergy2", "AceConsole-3.0", "AceEvent-3.0")
Comergy2.Movable = LibStub('LibMovable-1.0')
Comergy2.SM3 = LibStub("LibSharedMedia-3.0")
Comergy2:SetDefaultModuleLibraries("AceEvent-3.0")
Comergy2.Prism = LibStub("LibPrism-1.0")

local _, class = UnitClass("player")
Comergy2.class = class

local statusbars = {
    Patt   = "Patt",
    Flat   = "Flat",
    Smth   = "Smth",
    Alum   = "Alum",
    Pat2   = "Pat2",
    ['3D'] = "3d",
    Amry   = "Amry",
    Flat2  = "Flat2",
    Mini   = "Mini",
    Otra   = "Otra",
}

local sounds = {
    Piano1  = "combo1.ogg",
    Piano2  = "combo2.ogg",
    Piano3  = "combo3.ogg",
    Piano4  = "combo4.ogg",
    Piano5  = "combo5.ogg",
    Tick    = "energytick.ogg",
    Crit    = "critical.ogg",
}

for name, path in pairs(statusbars) do
    Comergy2.SM3:Register("statusbar", name, "Interface\\AddOns\\Comergy2\\textures\\"..path)
end

for name, path in pairs(sounds) do
    Comergy2.SM3:Register("sound", name, "Interface\\AddOns\\Comergy2\\sounds\\"..path)
end

local defaults = {
    profile = {
        hideCombat = false,
        showNotFull = true,
        critSound = "None",
        length = 224,
        bgColor = { 0.0, 0.0, 0.0, 0.7 },
        spacing = 4,
        frameStrata = "HIGH",
        fadingDelay = 3,
        vertical = false,
        flip = false,
        texture = "3D",
        fontSize = 14,
        textColor = { 1, 1, 1 },
        font = "Friz Quadrata TT",
        centerText = true,
        aboveFrame = true,
        playerHeight = 1,
        targetHeight = 1,
        position = {
            xOffset = 0,
            yOffset = -200,
            pointTo = "CENTER",
            pointFrom = "CENTER",
        }
    }
}

function Comergy2:OnInitialize()
    self:RegisterChatCommand("cmg2", "ProcessSlashCommand")

    self.db = LibStub("AceDB-3.0"):New("Comergy2DB", defaults)
    self.db.RegisterCallback(self, "OnProfileChanged", "RefreshConfig")
    self.db.RegisterCallback(self, "OnProfileCopied", "RefreshConfig")
    self.db.RegisterCallback(self, "OnProfileReset", "RefreshConfig")
    self.options.args.profiles = LibStub('AceDBOptions-3.0'):GetOptionsTable(self.db)

    local LibDualSpec = LibStub('LibDualSpec-1.0')
    LibDualSpec:EnhanceDatabase(self.db, "Comergy2")
    LibDualSpec:EnhanceOptions(self.options.args.profiles, self.db)

    self.frame = CreateFrame("Frame", "Comergy2_MainFrame", UIParent)
    self.frame:SetPoint("CENTER", 0, -200)

    self.frame.bg = self.frame:CreateTexture(nil, "BACKGROUND")
    self.frame.bg:SetAllPoints()

    local clr = self.db.profile.bgColor
    self.frame.bg:SetTexture(unpack(clr))

    self.frame:SetScript("OnUpdate",
        function(...)
            self:OnUpdate(...)
        end)

    self.frames = {}

    self:CreateAnimations()
    self:UpdateAnimations()
    self:Resize()
    self.Movable.RegisterMovable(self, self.frame, self.db.profile.position, "Comergy2Main")
end

function Comergy2:RefreshConfig()
    --noinspection UnusedDef
    for name, module in self:IterateModules() do
        module:RefreshConfig()
    end
    self:Resize()
    self:UpdateAnimations()
end

function Comergy2:OnEnable()
    self.frame:Show()
    self.SM3.RegisterCallback(self, "LibSharedMedia_Registered", "SM3_Update")
    self.SM3.RegisterCallback(self, "LibSharedMedia_SetGlobal", "SM3_Update")
    self:RegisterEvent("PLAYER_REGEN_DISABLED")
    self:RegisterEvent("PLAYER_REGEN_ENABLED")
    self:Print("Welcome to the Comergy2 Beta. Please leave any feedback or suggestions on WowAce, Curse, or SolidICE.")
end

function Comergy2:OnDisable()
    self.frame:Hide()
end

function Comergy2:OnUpdate()

    if self.db.profile.hideCombat and not InCombatLockdown() and
            not (self.db.profile.showNotFull and not self:IsStable()) then
        if self.fade and not self.fade:IsPlaying() and self.frame:GetAlpha() > 0 then
            self.fade:Play()
        end
    else
        self.frame:SetAlpha(1)
    end
end

function Comergy2:PLAYER_REGEN_DISABLED()
    if self.fade and self.fade:IsPlaying() then
        self.fade:Stop()
    end
    self.frame:SetAlpha(1)
end

function Comergy2:PLAYER_REGEN_ENABLED()
    if self.db.profile.hideCombat then
        if self.fade and not self.fade:IsPlaying() then
            self.fade:Play()
        end
    end
end

function Comergy2:CreateAnimations()
    local group = self.frame:CreateAnimationGroup(self:GetName().."_FadeAnimation")
    local alpha = group:CreateAnimation("Alpha")
    alpha:SetChange(-1)
    alpha:SetDuration(1)
--    alpha:SetStartDelay(self.db.profile.fadingDelay)
    --noinspection UnusedDef
    group:SetScript("OnFinished", function(...)
        self.frame:SetAlpha(0)
    end)
    group.alpha = alpha
    self.fade = group
end

function Comergy2:UpdateAnimations()
    self.fade.alpha:SetStartDelay(self.db.profile.fadingDelay)
end

--noinspection UnusedDef
function Comergy2:ProcessSlashCommand(input)
    input = input:trim()

    -- Settings menu
    if input == "" then
        self.ACD:Open("Comergy2")

    elseif input == "enable" then
        self:Enable()

    elseif input == "disable" then
        self:Disable()

    -- Toggle movable
    elseif input == "toggle" then
        self:Toggle()

    end
end

function Comergy2:IsStable()
    local stable = true
    --noinspection UnusedDef
    for name, module in self:IterateModules() do
        stable = stable and module:IsStable()
    end
    return stable
end

function Comergy2:Toggle()

    if self.Movable.IsLocked(self) then
        self.Movable.Unlock(self)

    else
        self.Movable.Lock(self)

    end
end

function Comergy2:ResetPosition()
    self.Movable.ResetMovableLayout()
end

function Comergy2:Resize()

    self.frame:SetWidth(self.db.profile.length)
    self.frame:SetScale(1)

    local h = self.db.profile.spacing

--    for i, frame in ipairs(self.frames) do
--
--        if i == 1 then
--            frame:SetPoint("BOTTOM", self.frame, "BOTTOM", 0, self.db.profile.spacing/2)
--
--        else
--            frame:SetPoint("BOTTOM", self.frames[i - 1], "TOP")
--
--        end
--        h = h + frame.h
--
--    end

    local cur = self.frames.first

    while cur ~= nil do
        h = h + cur.h
        cur = cur.next
    end

    self.frame:SetHeight(h)
    self.frame:SetFrameStrata(self.db.profile.frameStrata)

end

function Comergy2:IsFrameRegistered(handle)
    return self.frames[handle] and true or false
end

function Comergy2:PrintRegisteredFrames()
    self:Print(">>>Start<<<")
    for name, frame in pairs(self.frames) do
        local n = tostring(frame):len()
        local str = name..': '..tostring(frame):sub(n-2)
        --noinspection StringConcatenationInLoops
        str = str..' P: '..tostring(frame.prev):sub(n-2)
        --noinspection StringConcatenationInLoops
        str = str..' N: '..tostring(frame.next):sub(n-2)
        --noinspection StringConcatenationInLoops
        str = str..' X: '..frame.priority
        self:Print(str)
    end
end

function Comergy2:PrintRegisteredFramePoints()
    self:Print(">>>Start<<<")
    for name, frame in pairs(self.frames) do
        self:Print()
        self:Print(name, frame, frame.h)
        for i=1, frame:GetNumPoints() do
            self:Print(frame:GetPoint(i))
        end
--        local n = tostring(frame):len()
--        local str = name..': '..tostring(frame):sub(n-2)
--        str = str..' P: '..tostring(frame.prev):sub(n-2)
--        str = str..' N: '..tostring(frame.next):sub(n-2)
--        str = str..' X: '..frame.priority
--        self:Print(str)
    end
end

function Comergy2:UnregisterFrame(handle)
    if self.frames[handle] then

        local frame = self.frames[handle]
        frame:ClearAllPoints()

        if frame.next then
            frame.next.prev = frame.prev
        else
            self.frames.last = frame.prev
        end

        if frame.prev then
            frame.prev.next = frame.next
        else
            self.frames.first = frame.next
        end

        if frame.next and frame.prev then
            frame.next:SetPoint("BOTTOM", frame.prev, "TOP")
        elseif frame.next then
            frame.next:SetPoint("BOTTOM", self.frame, "BOTTOM", 0, self.db.profile.spacing/2)
        end

        frame.next = nil
        frame.prev = nil
        frame:Hide()
        self.frames[handle] = nil

    end

    self:Resize()
--    self:PrintRegisteredFramePoints()
end

function Comergy2:RegisterFrame(frame, handle)

    self.frames[handle] = frame
    frame:Show()
    frame:SetPoint("LEFT", self.frame)
    frame:SetPoint("RIGHT", self.frame)

    if not self.frames.first then
        self.frames.first = frame
        self.frames.last = frame
        frame:SetPoint("BOTTOM", self.frame, "BOTTOM", 0, self.db.profile.spacing/2)
    else
        local cur = self.frames.last
        while cur ~= nil and cur.priority > frame.priority do
            cur = cur.prev
        end
        if cur == nil then
            frame.next = self.frames.first
            frame.next.prev = frame
            self.frames.first = frame
            frame:SetPoint("BOTTOM", self.frame, "BOTTOM", 0, self.db.profile.spacing/2)
        else
            frame.next = cur.next
            if cur.next then
                cur.next.prev = frame
            end
            cur.next = frame
            frame.prev = cur
            frame:SetPoint("BOTTOM", frame.prev, "TOP")
        end
        if frame.next ~= nil then
            frame.next:SetPoint("BOTTOM", frame, "TOP")
        else
            self.frames.last = frame
        end
    end

    self:Resize()
--    self:PrintRegisteredFramePoints()

--    self:Print(">>>START<<<")
--    for name, frame in pairs(self.frames) do
--        self:Print(name, frame, frame.prev, frame.next, frame.priority)
--    end
end

--noinspection UnusedDef
function Comergy2:SM3_Update( callback, type, handle )

    if type == "statusbar" and handle == self.db.profile.texture then
        --noinspection UnusedDef
        for name, module in self:IterateModules() do
            module:UpdateTextures()
        end
    end

end
