--
-- Created by IntelliJ IDEA.
-- User: admin
-- Date: 7/07/2014
-- Time: 3:34 PM
-- To change this template use File | Settings | File Templates.
--
local Ace = LibStub("AceAddon-3.0")
local Comergy2 = Ace:GetAddon("Comergy2")

--noinspection UnusedDef
function Comergy2:GetProtoType_Health()

    --noinspection UnusedDef,UnusedDef
    return {

        TARGET = "player",

        defaults = {
            profile = {
                target = {
                    enabled = true,
                    color0 = { 1.0, 0.0, 0.0 },
                    color1 = { 0.0, 1.0, 0.0 },
                    height = 2,
                    bgAlpha = 0.2,
                    text = true,
                    flash = true,
                    threshold = 35,
                    priority = 4,
                },
                player = {
                    enabled = true,
                    color0 = { 1.0, 0.0, 0.0 },
                    color1 = { 0.0, 1.0, 0.0 },
                    height = 2,
                    bgAlpha = 0.2,
                    text = true,
                    flash = true,
                    threshold = 35,
                    priority = 1,
                },
                focus = {
                    enabled = true,
                    color0 = { 1.0, 0.0, 0.0 },
                    color1 = { 0.0, 1.0, 0.0 },
                    height = 2,
                    bgAlpha = 0.2,
                    text = true,
                    flash = true,
                    threshold = 35,
                    priority = 5,
                },
            }
        },

        OnInitialize = function(self)
            self.db = Comergy2.db:RegisterNamespace("Module_Health", self.defaults)
            self.target = {
                key = self:GetName().."_Target",
            }
            self.player = {
                key = self:GetName().."_Player",
            }
            self.focus = {
                key = self:GetName().."_Focus",
            }

            self:CreateFrames()

            self:ResizeFrames()

            self:CreateBars()
            self:SetMax()

            self:UpdateTextures()
--            self:CreateAnimations()

            local options = self:GetOptions()
            options.handler = self

            Comergy2.options.args.health = options
        end,

        SetMax = function(self, unit)
            if unit ~= nil then
                self[unit].max = UnitHealthMax(unit)
                self[unit].bar:SetMinMaxValues(0, self[unit].max)
            else
                self.target.max = UnitHealthMax("target")
                self.target.bar:SetMinMaxValues(0, self.target.max)
                self.player.max = UnitHealthMax("player")
                self.player.bar:SetMinMaxValues(0, self.player.max)
                self.focus.max = UnitHealthMax("focus")
                self.focus.bar:SetMinMaxValues(0, self.focus.max)
            end
        end,

        UpdateTextures = function(self)
            local tex = Comergy2.SM3:Fetch("statusbar", Comergy2.db.profile.texture)
            local units = {"target", "player", "focus" }
            for i = 1, 3 do
                local unit = units[i]
                self[unit].bar:SetStatusBarTexture(tex)
                self[unit].bar.bg:SetTexture(tex)
            end
        end,

        OnEnable = function(self)
            self:RegisterEvent("PLAYER_TARGET_CHANGED")
            self:RegisterEvent("PLAYER_FOCUS_CHANGED")
        end,

        OnDisable = function(self)
            self:UnregisterEvent("PLAYER_TARGET_CHANGED")
            self:UnregisterEvent("PLAYER_FOCUS_CHANGED")
        end,

        PLAYER_TARGET_CHANGED = function(self, cause)
            self:SetMax("target")
        end,

        PLAYER_FOCUS_CHANGED = function(self)
            self:SetMax("focus")
        end,

        --    Name Conflicts?
        CreateAnimations = function(self)
            local group = self.frame:CreateAnimationGroup(self:GetName().."_FlashAnimation")
            local alpha = group:CreateAnimation("Alpha")
            alpha:SetChange(-1)
            alpha:SetDuration(0.2)
            group:SetLooping("BOUNCE")
            group:SetScript("OnLoop", function(...)
                self:OnLoop(...)
            end)
            self.flash = group
        end,

        OnLoop = function(self, widget, loopState)
            if loopState == "REVERSE" then
                self.flashTimes = self.flashTimes - 1
                if self.flashTimes < 1 then -- allow for negative flashTimes
                self.flash:Finish()
                end
            end
        end,

        OnUpdate = function(self, unit, ...)
            self:UpdateHealth(unit)
        end,

        UpdateHealth = function(self, unit)
            local health = UnitHealth(unit)
            self[unit].health = health
            self[unit].bar:SetValue(health)

            local x = self[unit].max == 0 and 0 or health/self[unit].max

            local r0, g0, b0 = unpack(self.db.profile[unit]["color0"])
            local r1, g1, b1 = unpack(self.db.profile[unit]["color1"])
            --noinspection UnusedDef
            local hex, r, g, b = Comergy2.Prism:Gradient('hsv', r0, r1, g0, g1, b0, b1, x)
            self[unit].bar:SetStatusBarColor(r, g, b)
            self[unit].bar.bg:SetVertexColor(r, g, b, self.db.profile[unit].bgAlpha)
        end,

        UpdateFlash = function(self)
            if self.db.profile.flash and self.pwr == self.max then
                if self.flash and not self.flash:IsPlaying() and self.flashTimes == -1 then
                    self.flashTimes = 2
                    self.flash:Play()
                end
            else
                self.flashTimes = -1
            end
        end,

        CreateFrames = function(self)
            local units = {"target", "player", "focus" }
            for i = 1, 3 do
                local unit = units[i]
                self[unit].frame = CreateFrame("Frame", "Comergy2_"..unit.."Health", Comergy2.frame)
                self[unit].frame:Show()

--                self[unit].frame:SetPoint("LEFT")
--                self[unit].frame:SetPoint("RIGHT")

                self[unit].frame:SetScript("OnUpdate",
                    function(...)
                        self:OnUpdate(unit, ...)
                    end)

                self[unit].frame.priority = self.db.profile[unit].priority
            end

            self:ResizeFrames()
            if self.db.profile.target.enabled then
                Comergy2:RegisterFrame(self.target.frame, self.target.key)
            end
            if self.db.profile.player.enabled then
                Comergy2:RegisterFrame(self.player.frame, self.player.key)
            end
            if self.db.profile.focus.enabled then
                Comergy2:RegisterFrame(self.focus.frame, self.focus.key)
            end
        end,

        ResizeFrames = function(self)
            local units = {"target", "player", "focus" }
            for i = 1, 3 do
                local unit = units[i]

                self[unit].frame.h = self.db.profile[unit].height + Comergy2.db.profile.spacing
                self[unit].frame:SetHeight(self[unit].frame.h)
            end
            Comergy2:Resize()
        end,

        CreateBars = function(self)
            local units = {"target", "player", "focus" }
            for i = 1, 3 do
                local unit = units[i]

                local bar = CreateFrame("StatusBar", "Comergy2_"..unit.."HealthBar", self[unit].frame)
                bar:SetMinMaxValues(0, 1)


                bar:SetPoint("TOPLEFT", Comergy2.db.profile.spacing, -Comergy2.db.profile.spacing/2)
                bar:SetPoint("BOTTOMRIGHT", -Comergy2.db.profile.spacing, Comergy2.db.profile.spacing/2)

                bar.bg = bar:CreateTexture(nil, "BACKGROUND", 3)
                bar.bg:SetAllPoints()

                self[unit].bar = bar
            end
        end,

        RefreshConfig = function(self)
            self:ResizeFrames()

            self:UpdateTextures()
--            self:CreateAnimations()
        end,

        IsStable = function(self) return true end,

        Get = function(self, info)
--            for key, value in pairs(info) do
--                print(key, value)
--            end
            return self.db.profile[info[#info-1]][info[#info]]
        end,

        Set = function(self, info, value)
            self.db.profile[info[#info-1]] [info[#info]] = value
            self:RefreshConfig()
        end,

--        SetPosition = function(self, info, value)
--            self.db.profile["position"][info[#info]] = value
--            self:RefreshConfig()
--        end,
--
--        GetPosition = function(self, info)
--            return self.db.profile["position"][info[#info]]
--        end,

        GetColor = function(self, info)
            local color = self.db.profile[info[#info-1]][info[#info]] or {1, 1, 1, 1}
            return color[1], color[2], color[3], color[4]
        end,

        SetColor = function(self, info, r, g, b, a)
            self.db.profile[info[#info-1]][info[#info]] = {r, g, b, a}
            self:RefreshConfig()
        end,

--        GetColorText = function(self, info)
--            local idx = info[#info]:sub(1, -5)
--            local color = self.db.profile[idx] or {1, 1, 1, 1}
--            return format('%02X%02X%02X', color[1]*255, color[2]*255, color[3]*255)
--        end,
--
--        SetColorText = function(self, info, value)
--            local a = 1
--            local r = tonumber(value:sub(1,2), 16) / 255
--            local g = tonumber(value:sub(3,4), 16) / 255
--            local b = tonumber(value:sub(5,6), 16) / 255
--            self.db.profile[info[#info]:sub(1, -5)] = {r, g, b, a}
--            self:RefreshConfig()
--        end,

        --noinspection UnusedDef
        GetEnable = function(self, info)
            return self.db.profile[info[#info-1]].enabled
--            return Comergy2:IsFrameRegistered(self[info[#info-1]].key)
        end,

        --noinspection UnusedDef
        SetEnable = function(self, info, value)
            self.db.profile[info[#info-1]].enabled = value

            if value then
                Comergy2:RegisterFrame(self[info[#info-1]].frame, self[info[#info-1]].key)
            else
                Comergy2:UnregisterFrame(self[info[#info-1]].key)
                --                self:Disable()
--                self[info[#info-1]].frame.h = 0
--                self[info[#info-1]].frame:SetHeight(0)
--                self[info[#info-1]].frame:Hide()
--                Comergy2:Resize()
            end
        end,

        GetOptions = function(self)
            local opts = {
                type = 'group',
                name = 'Health',
                order = 4,
                disabled = false,
                args = {},
            }

            local units = {"Player", "Target", "Focus" }

            for i = 1, 3 do
                local unit = units[i]

                opts.args[unit:lower()] = {
                    type = 'group',
                    inline = true,
                    name = unit,
                    order = i,
                    args = {
                        enable = {
                            type = 'toggle',
                            name = 'Enable',
                            desc = 'Show a bar tracking the '..unit..'\'s health',
                            order = 1,
                            set = 'SetEnable',
                            get = 'GetEnable',
                            width = 'half',
                        },
                        height = {
                            type = 'range',
                            name = 'Height',
                            desc = 'Height of the health bar',
                            min = 1,
                            max = 1000,
                            softMax = 30,
                            step = 1,
                            width = 'normal',
                            order = 6,
                        },
                        color0 = {
                            type = 'color',
                            name = 'Empty',
                            desc = 'Color of bar when empty',
                            set = 'SetColor',
                            get = 'GetColor',
                            order = 4,
                            width = 'half',
                        },
                        color1 = {
                            type = 'color',
                            name = 'Full',
                            desc = 'Color of bar when full',
                            set = 'SetColor',
                            get = 'GetColor',
                            order = 5,
                            width = 'half',
                        },
                        flash = {
                            type = 'toggle',
                            name = 'Flash',
                            desc = 'Flash bar when below threshold',
                            order = 2,
                            disabled = true,
                            width = 'half',
                        },
                        threshold = {
                            type = 'range',
                            name = 'Threshold',
                            desc = 'Threshold for bar flash',
                            min = 0,
                            max = 100,
                            step = 1,
                            disabled = true,
                            width = 'normal',
                            order = 3,
                        },
                    }
                }
            end

            return opts
        end
    }
end
